<?php
namespace Zeedhi\DevOps\Model\Servermanager\Repositories;

use \Doctrine\ORM\EntityRepository;
use Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource;
use Zeedhi\DevOps\Model\Servermanager\Entities\ProductServer;
use Zeedhi\DevOps\Util\Utilities as DevOpsUtilities;

class Server extends EntityRepository {

    /**
     * @param $filters
     * @param null $organizationId
     * @return array
     */
    public function findServersWithFilter($filters, $organizationId = null, $limit = null, $offset = null) {
        $query = <<<SQL
			SELECT *
			  FROM (SELECT SR.ID AS SERVER_ID, SR.NAME, SR.WSEMAIL, PS.ID, PS.CREATED,
                           PS.CREATED_BY, PS.UPDATED, PS.UPDATED_BY, PS.IS_ACTIVE,
                           PS.ORGANIZATION_ID, PS.USETYPE, PS.BASE_URL, PS.PATH,
                           NVL(PS.BACKUP_LIMIT, SR.BACKUP_LIMIT) AS BACKUP_LIMIT,
                           PR.ID AS PRODUCT_ID, PR.NAME AS PRODUCT_NAME,
                           RT.NAME AS RESOURCE_NAME, SR.IS_WINDOWS, SR.IS_PRODUCTION
                      FROM SERVER SR
                INNER JOIN PRODUCT_SERVER PS ON PS.SERVER_ID = SR.ID
                 LEFT JOIN RESOURCE_T RT ON RT.ID = SR.RESOURCE_T_ID
                INNER JOIN PRODUCT PR ON PR.ID = PS.PRODUCT_ID
                     WHERE SR.ORGANIZATION_ID = :ORGANIZATION
                     ORDER BY PS.ID)
             WHERE 1 =1
             $filters
SQL;

        $params = array(
            'ORGANIZATION' => $organizationId
        );
        $query = DevOpsUtilities::getPaginatedQuery($query, $limit, $offset);
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findScriptsByServe($serverId, $organizationId = null) {
        $query = <<<SQL
			SELECT PR.ID AS PRODUCT_ID, PR.NAME AS PRODUCT_NAME,PS.USETYPE AS TYPE
              FROM SERVER SR
              JOIN PRODUCT_SERVER PS ON (PS.SERVER_ID = SR.ID)
              JOIN PRODUCT PR ON (PR.ID = PS.PRODUCT_ID)
             WHERE PR.IS_ACTIVE = 'Y'
               AND SR.ID = :SERVER
               AND SR.ORGANIZATION_ID = :ORGANIZATION
             ORDER BY PR.ID
SQL;
        $params = array(
            'SERVER' => $serverId,
            'ORGANIZATION' => $organizationId
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    /**
     * @param $organizationId
     * @param $filters
     * @return array
     */
    public function findServers($organizationId, $filters, $limit = null, $offset = null) {
        $query = <<<SQL
        SELECT *
          FROM (SELECT SR.BACKUP_LIMIT, SR.DESCRIPTION, SR.ID, SR.IS_ACTIVE,
                       SR.NAME, SR.ONLY_APPROVED_SCRIPTS, SR.ORGANIZATION_ID,
                       SR.RESOURCE_T_ID, SR.WSEMAIL, RT.NAME AS RESOURCE_NAME,
                       SR.IS_WINDOWS, SR.IS_PRODUCTION
                  FROM SERVER SR
             LEFT JOIN RESOURCE_T RT ON RT.ID = SR.RESOURCE_T_ID
                                    AND RT.CLASS = :SPONSOR_RESOURCE_CLASS
                 WHERE SR.ORGANIZATION_ID = $organizationId
                 AND SR.IS_ACTIVE = 'Y')
         WHERE 1 = 1
      $filters
      ORDER BY ID
SQL;
        $query = DevOpsUtilities::getPaginatedQuery($query, $limit, $offset);
        $params = array(
            'SPONSOR_RESOURCE_CLASS' => Resource::CLASS_SPONSOR
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    /**
     * @param $productId
     * @param $type
     * @return array
     */
    public function findServerByProductAndType($productId, $type, $filterProductServerStatus = null) {
        if($filterProductServerStatus == 'Y'){
            $filterProductServerStatus = "AND PS.IS_ACTIVE <> 'N'";
        }else {
            $filterProductServerStatus = "";
        }
        
        $query = <<<SQL
			SELECT SR.ID AS SERVER_ID, SR.NAME, SR.WSEMAIL, PS.USETYPE,
			       PS.BASE_URL, PS.PATH, PR.ID AS PRODUCT_ID,
			       PS.ID AS PRODUCT_SERVER_ID, RT.NAME AS CUSTOMER,
			       NVL(PS.BACKUP_LIMIT, SR.BACKUP_LIMIT) AS BACKUP_LIMIT, PS.IS_ACTIVE
              FROM SERVER SR
        INNER JOIN PRODUCT_SERVER PS ON PS.SERVER_ID = SR.ID
         LEFT JOIN RESOURCE_T RT ON RT.ID = SR.RESOURCE_T_ID
        INNER JOIN PRODUCT PR ON PR.ID = PS.PRODUCT_ID
             WHERE PR.ID = :PRODUCT
               AND PS.USETYPE = :TYPE $filterProductServerStatus
             ORDER BY PS.ID
SQL;
        $params = array('PRODUCT' => $productId, 'TYPE' => $type);

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    /**
     * @param $productId
     * @param $customerCode
     * @param $filterQuery
     * @return array
     */
    public function findCustomerProductServers($productId, $customerCode, $filterQuery) {
        $query = <<<SQL
        SELECT *
          FROM (SELECT SR.ID AS SERVER_ID, SR.NAME, PS.ID, PS.PATH, SR.WSEMAIL,
                       PS.ID AS PRODUCT_SERVER_ID, PS.PRODUCT_ID,
                       PC.ID AS PACKAGE_ID, PC.PATH_TO_DEPLOY AS PRODUCT_PATH,
                       NVL(PS.BACKUP_LIMIT, SR.BACKUP_LIMIT) AS BACKUP_LIMIT,
                       SR.IS_WINDOWS AS SERVER_IS_WINDOWS, PS.EXEC_SCRIPTS_AUTOMATIC,
                       SR.IS_REQUIRED_TICKET AS SERVER_REQUIRES_TICKET
                  FROM PRODUCT_SERVER PS
                 INNER JOIN SERVER SR ON SR.ID = PS.SERVER_ID
                 INNER JOIN RESOURCE_T RT ON RT.ID = SR.RESOURCE_T_ID
                  LEFT JOIN PACKAGE PC ON PC.ID = PS.PACKAGE_ID
                 WHERE PS.PRODUCT_ID = :PRODUCT_ID
                   AND PS.IS_ACTIVE = :IS_ACTIVE
                   AND SR.IS_ACTIVE = :IS_ACTIVE
                   AND PS.USETYPE   = :PRODUCTION_TYPE
                   AND RT.EXTERNAL_CODE = :CUSTOMER_CODE)
         WHERE 1 = 1
               $filterQuery
SQL;
        $params = array(
            'PRODUCT_ID'      => $productId,
            'CUSTOMER_CODE'   => $customerCode,
            'IS_ACTIVE'       => ProductServer::IS_ACTIVE_POSITIVE,
            'PRODUCTION_TYPE' => ProductServer::TYPE_PRODUCTION
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    /**
     * @param $customerCode
     * @param $filterQuery
     * @return array
     */
    public function findAvailableCustomerServers($customerCode, $filterQuery) {
        $query = <<<SQL
        SELECT *
          FROM (SELECT SR.*
                  FROM SERVER SR
                 INNER JOIN RESOURCE_T RT ON RT.ID = SR.RESOURCE_T_ID
                 WHERE RT.EXTERNAL_CODE = :CUSTOMER_CODE)
         WHERE 1 = 1
               $filterQuery
SQL;
        $params = array(
            'CUSTOMER_CODE' => $customerCode,
            'IS_ACTIVE'     => ProductServer::IS_ACTIVE_POSITIVE
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }
    
}