<?php
namespace Teknisa\Model\Gestaopessoas\Repositories;

use HCM\Util\Repositories;
use HCM\Util\DateUtil;

class FpaCalculofolha extends \Doctrine\ORM\EntityRepository {
    
    public function retornaCalculoFolha($nrorg, $nrtipomovimento, $nrTpModalidadeCalc, $nrOcorrenciaCalc, $dtInicial, $dtFinal){
        
        $fpacalculofolha = Repositories::FPA_CALCULOFOLHA;
            $params = array('nrorg'=>$nrorg, 'nrtipomoviment'=>$nrtipomovimento, 'nrtpmodalidcal'=>$nrTpModalidadeCalc,
         'dtinicial'=> $dtInicial->format('d/m/Y'), 'dtfinal'=> $dtFinal->format('d/m/Y'));
        if($nrOcorrenciaCalc){
$sql = <<<SQL
SELECT c FROM $fpacalculofolha c
WHERE c.nrorg           = :nrorg
  AND c.nrtipomoviment = :nrtipomoviment
  AND c.nrtpmodalidcal      = :nrtpmodalidcal
  AND c.nrocorrecal      = :nrocorrecal
  AND c.dtocorrencia between :dtinicial and :dtfinal
SQL;
        $params['nrocorrecal'] = $nrOcorrenciaCalc;
        
        }else{
        
$sql = <<<SQL
SELECT c FROM $fpacalculofolha c
WHERE c.nrorg = :nrorg
  AND c.nrtipomoviment = :nrtipomoviment
  AND c.nrtpmodalidcal  = :nrtpmodalidcal
  AND c.nrocorrecal >= 0
  AND c.dtocorrencia between :dtinicial and :dtfinal
SQL;
        }
        
        $result = $this->getEntityManager()->createQuery($sql)->execute($params);
        
        return $result;
    }
    
    public function retornaCalculoFolhaTodasOcorrencias($nrorg, $nrTipoMovimento, $nrTpModalidadeCalc, $dtInicial, $dtFinal){
        
        $dtInicial = DateUtil::getDataDeString($dtInicial->format('d/m/Y'), DateUtil::FORMATO_BRASILEIRO, true);
        $dtFinal = DateUtil::getDataDeString($dtFinal->format('d/m/Y'), DateUtil::FORMATO_BRASILEIRO, true);
        
        $params = array('nrorg'=>$nrorg, 'nrtipomoviment'=>$nrTipoMovimento, 'nrtpmodalidcal'=>$nrTpModalidadeCalc,
        'dtinicial'=>$dtInicial, 'dtfinal'=>$dtFinal);
        
        $fpacalculofolha = Repositories::FPA_CALCULOFOLHA;
        
$sql = <<<SQL
SELECT c FROM $fpacalculofolha c
WHERE c.nrorg           = :nrorg
  AND c.nrtipomoviment = :nrtipomoviment
  AND c.nrtpmodalidcal      = :nrtpmodalidcal
  AND c.nrocorrecal > 0
  AND (c.dtocorrencia between :dtinicial AND :dtfinal)
SQL;
        $result = $this->getEntityManager()->createQuery($sql)->execute($params);
        return $result;
    }

    public function getCalculoFolha($nrorg, $nrtipomovimento, $nrmodalidade, $nrocorrecalc, $dtocorrencia) {
        
        $params = array('nrorg'=>$nrorg, 'nrtipomoviment'=>$nrtipomovimento, 'nrtpmodalidcal'=>$nrmodalidade,
        'nrocorrecal'=>$nrocorrecalc, 'dtocorrencia'=>$dtocorrencia->format('d/m/Y'));
        
        $fpacalculofolha = Repositories::FPA_CALCULOFOLHA;
        
$sql = <<<SQL
SELECT c FROM $fpacalculofolha c
WHERE c.nrorg           = :nrorg
  AND c.nrtipomoviment = :nrtipomoviment
  AND c.nrtpmodalidcal      = :nrtpmodalidcal
  AND c.nrocorrecal = :nrocorrecal
  AND (c.dtocorrencia between :dtocorrencia and :dtocorrencia)
SQL;

        $result = $this->getEntityManager()->createQuery($sql)->execute($params);
        return $result;
    }
    
    public function getCalculoFolhaAndTpCalcMov($nrorg, $nrcalculoFolha, $nrTipoCalculo) {
        
        $params = array('nrorg'=> $nrorg, 'nrcalculofolha'=>$nrcalculoFolha, 'nrtipocalculo' => $nrTipoCalculo);
        
        $fpacalculofolha = Repositories::FPA_CALCULOFOLHA;
        $fpaTpCalcMov = Repositories::FPA_TPCALCMOV;
        
$sql = <<<SQL

SELECT  cf, tcm.nmtpcalcmov
                FROM $fpacalculofolha cf
                INNER JOIN $fpaTpCalcMov tcm
                    WITH cf.nrtipomoviment = tcm.nrtipomoviment
                WHERE cf.nrorg = :nrorg
                        AND cf.nrcalculofolha = :nrcalculofolha
                    AND tcm.nrtipocalculo = :nrtipocalculo
SQL;

        $result = $this->getEntityManager()->createQuery($sql)->execute($params);
        return $result;
    }
    
    public function getCalculoFolhaAndTpCalc($nrorg, $nrcalculoFolha) {
        
        $params = array('nrorg'=>$nrorg, 'nrcalculofolha'=>$nrcalculoFolha);
        
        $fpacalculofolha = Repositories::FPA_CALCULOFOLHA;
        $fpaTipoCalculo = Repositories::FPA_TIPOCALCULO;
        
$sql = <<<SQL

SELECT c, tc.nmtipocalculo FROM $fpacalculofolha c
JOIN $fpaTipoCalculo tc
WITH c.nrtipomoviment = tc.nrtipocalculo
                WHERE c.nrorg = :nrorg
                        AND c.nrcalculofolha = :nrcalculofolha
SQL;

        $result = $this->getEntityManager()->createQuery($sql)->execute($params);
        return $result;
    }
    
    /**
     * Busca no banco de dados os valores mensais de remuneração e horas extras
     * @param $nrorg
     * @param $dtocorrencia
     * @param $nrtpmovimento
     * @return array|bool
     */
    public function getNrCalculo($nrorg, $dtocorrencia,$nrtipomoviment)
    {

        $params = array('nrorg'=>$nrorg, 'dtocorrencia'=>$dtocorrencia, 'nrtipomoviment'=>$nrtipomoviment);
        
        
        $fpacalculofolha = Repositories::FPA_CALCULOFOLHA;
        
        $sql = <<<SQL

        SELECT c.nrcalculofolha
FROM $fpacalculofolha c 
WHERE c.nrorg       =:nrorg
AND c.dtocorrencia  =:dtocorrencia
AND c.nrtipomoviment=:nrtipomoviment
AND c.nrtpmodalidcal=1
AND c.nrocorrecal   =1
SQL;
        $result = $this->getEntityManager()->createQuery($sql)->execute($params);

        if($result && isset($result[0]) && $result[0]['nrcalculofolha']) $result = $result[0]['nrcalculofolha'];

        return $result;
    }

    public function getSomaItemCalculo($nrvinculom, $nrorg, $data_inicial, $data_final, $nreventom, $nrtipomoviment, $dtmescompetenc)
    {
        $db = \Setforms2\Shared\Registry::getEntityManagerConnection();
        
        $data_inicial = $data_inicial->format('d/m/Y');
        $data_final = $data_final->format('t/m/Y');
        $dtmescompetenc = $dtmescompetenc->format('d/m/Y');

        $params = array(
            'p_nrorg'          => $nrorg, 
            'p_dtmescompetenc' => $dtmescompetenc, 
            'p_nrvinculom'     => $nrvinculom, 
        );
        

        $sql = <<<SQL
SELECT SUM(DECODE(FPA_EVENTOH.IDMEDIAVALREF, 'REFERENCIA', FPA_ITECALCFOLHA.VRREFFOLHA, FPA_ITECALCFOLHA.VREVENTOFOLHA)) VALOR,
    FPA_ITECALCFOLHA.NRVINCULOM,
    FPA_ITECALCFOLHA.NRORG

FROM FPA_CALCULOFOLHA

INNER JOIN FPA_ITECALCFOLHA
    ON FPA_ITECALCFOLHA.NRCALCULOFOLHA = FPA_CALCULOFOLHA.NRCALCULOFOLHA
    AND FPA_ITECALCFOLHA.NRORG = FPA_CALCULOFOLHA.NRORG

INNER JOIN FPA_EVENTOH
    ON FPA_ITECALCFOLHA.NREVENTOM = FPA_EVENTOH.NREVENTOM
    AND (FPA_ITECALCFOLHA.NRORG = FPA_EVENTOH.NRORG OR FPA_EVENTOH.NRORG = 0)
    AND FPA_EVENTOH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) DTMESCOMPETENC
                        FROM FPA_EVENTOH MAX_EVENTOH
                        WHERE MAX_EVENTOH.NREVENTOM = FPA_EVENTOH.NREVENTOM
                          AND MAX_EVENTOH.NRORG = FPA_EVENTOH.NRORG
                          AND MAX_EVENTOH.DTMESCOMPETENC <= :p_dtmescompetenc)

WHERE FPA_ITECALCFOLHA.NREVENTOM IN ($nreventom)
    AND FPA_CALCULOFOLHA.NRTIPOMOVIMENT IN ($nrtipomoviment) --( 1, 5, 6, 9)
    AND FPA_CALCULOFOLHA.NRTPMODALIDCAL = 1
    AND FPA_CALCULOFOLHA.NROCORRECAL = 1
    AND (FPA_CALCULOFOLHA.DTOCORRENCIA BETWEEN TO_DATE('$data_inicial') AND TO_DATE('$data_final'))
    AND FPA_ITECALCFOLHA.NRVINCULOM = :p_nrvinculom
    AND FPA_ITECALCFOLHA.NRORG = :p_nrorg

GROUP BY FPA_ITECALCFOLHA.NRVINCULOM,
    FPA_ITECALCFOLHA.NRORG
SQL;

        $resultado = $db->fetchAll($sql,$params);

        return ($resultado)?$resultado[0]:false;

    }
    
    public function _temFeriasDefinitivoComp($nrVinc, $nrorgTrab, $dtMesCompetenc, $competAnterior){

        $db = \Setforms2\Shared\Registry::getEntityManagerConnection();
        
        $params = array(
            'P_NRORG'=>$nrorgTrab,
            'P_NRVINCULOM'=>$nrVinc,
            'P_DTMESCOMPETENC'=>$dtMesCompetenc,
            'P_COMPANTER'=>$competAnterior
        );

$sql = <<<SQL
    SELECT 1 AS TEMDEFINITIVO FROM FPA_ITECALCFOLHA ITE
    JOIN FPA_CALCULOFOLHA CALC 
        ON CALC.NRCALCULOFOLHA = ITE.NRCALCULOFOLHA
        AND CALC.NRORG = ITE.NRORG
        AND CALC.NRTIPOMOVIMENT = 2
        AND CALC.NROCORRECAL = 1
        AND CALC.NRTPMODALIDCAL = 1 
        AND (CALC.DTOCORRENCIA BETWEEN :P_COMPANTER AND :P_DTMESCOMPETENC)
    
    WHERE ITE.NRVINCULOM = :P_NRVINCULOM
        AND ITE.NRORG = :P_NRORG
        AND ITE.NREVENTOM = 3030
SQL;
        $result = $db->fetchAll($sql,$params);

        if(isset($result[0])) return $result[0];
        else return null;
    }
    

    
}